@@ -1,7 +1,7 @@
 # -*- perl -*-
 package Smokeping::Master;
 use Data::Dumper;
-use Storable qw(nstore_fd dclone fd_retrieve);
+use Storable qw(nstore dclone fd_retrieve);
 use strict;
 use warnings;
 use Fcntl qw(:flock);
@@ -110,48 +110,51 @@ sub save_updates {
     my $updates = shift;
     # name\ttime\tupdatestring
     # name\ttime\tupdatestring
-
+    my %u;
     for my $update (split /\n/, $updates){
         my ($name, $time, $updatestring) = split /\t/, $update;
-        my $file = slavedatadir($cfg) ."/${name}.${slave}.slave_cache";
         if ( ${name} =~ m{(^|/)\.\.($|/)} ){
             warn "Skipping update for ${name}.${slave}.slave_cache since ".
                  "you seem to try todo some directory magic here. Don't!";
-        } 
-        else {
-
-            for (my $i = 2; $i >= 0; $i--){
-                my $fh;
-                if ( open ($fh, '+>>' , $file) and flock($fh, LOCK_EX) ){
-                    my $existing = [];
-                    if (! -e $file) { # the reader unlinked it from under us
-                        flock($fh, LOCK_UN);
-                        close $fh;
-                        next;
-                    }
-                    seek $fh, 0, 0;
-                    if ( -s _ ){
-                        my $in = eval { fd_retrieve $fh };
-                        if ($@) { #error
-                            warn "Loading $file: $@";
-                        } else {
-		            $existing = $in;
-			};
-                    };
-                    push @{$existing}, [ $slave, $time, $updatestring ];
-                    nstore_fd($existing, $fh);		    
+        } else {
+            push @{$u{$name}}, [$time,$updatestring];
+        }
+    }
+    for my $name (sort keys %u){
+        my $file = slavedatadir($cfg) ."/${name}.${slave}.slave_cache";
+        for (my $i = 2; $i >= 0; $i--){
+            my $fh;
+            if ( open ($fh, '+>>' , $file) and flock($fh, LOCK_EX) ){
+                my $existing = [];
+                if (! -e $file) { # the reader unlinked it from under us
                     flock($fh, LOCK_UN);
                     close $fh;
-                    last;
-                } elsif ($i > 0) {
-                    warn "Could not lock $file ($!). Trying again $i more times.\n";
-                    close $fh;
-                    sleep rand(2);
                     next;
                 }
-                warn "Could not update $file, giving up for now.";
+                seek $fh, 0, 0;
+                if ( -s _ ){
+                    my $in = eval { fd_retrieve $fh };
+                    if ($@) { #error
+                        warn "Loading $file: $@";
+                    } else {
+                        $existing = $in;
+    			    };
+                };
+                map {
+                    push @{$existing}, [ $slave, $_->[0], $_->[1] ];
+                } @{$u{$name}};
+                nstore($existing, $file.$$);               
+               rename $file.$$,$file;
+                flock($fh, LOCK_UN);
                 close $fh;
+                last;
+            } elsif ($i > 0) {
+                warn "Could not lock $file ($!). Trying again $i more times.\n";
+                sleep rand(2);
+                next;
             }
+            warn "Could not update $file, giving up for now.";
+            close $fh;
         }
     }         
 };
@@ -210,8 +213,11 @@ sub get_secret {
             close $hand;
             return $1;
         }
-    } 
-    warn "WARNING: Opening $cfg->{Slaves}{secrets}: $!\n";    
+    } else { 
+        print "Content-Type: text/plain\n\n";
+        print "WARNING: Opening secrets file $cfg->{Slaves}{secrets}: $!\n";
+        return '__HORRIBLE_INLINE_SIGNALING__';
+    }
     return;
 }
 
@@ -227,6 +233,7 @@ sub answer_slave {
     my $q = shift;
     my $slave = $q->param('slave');
     my $secret = get_secret($cfg,$slave);
+    return if $secret eq '__HORRIBLE_INLINE_SIGNALING__';
     if (not $secret){
         print "Content-Type: text/plain\n\n";
         print "WARNING: No secret found for slave ${slave}\n";       
